package com.fjhx.service.stock.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.constants.StatusConstant;
import com.fjhx.entity.stock.Stock;
import com.fjhx.entity.water.WaterTag;
import com.fjhx.enums.InTypeEnum;
import com.fjhx.mapper.stock.StockMapper;
import com.fjhx.service.stock.StockService;
import com.fjhx.utils.BigDecimalUtil;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 库存表 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-09-22
 */
@Service
public class StockServiceImpl extends ServiceImpl<StockMapper, Stock> implements StockService {

    @Override
    public void edit(Enum<?> typeEnum, List<WaterTag> waterTagList) {

        // 标签按物料id分组，统计出改变件数和数量
        Map<Long, Stock> changeStockMap = waterTagList.stream().collect(Collectors.toMap(
                WaterTag::getMaterialId,
                item -> {
                    Stock stock = new Stock();
                    stock.setMaterialId(item.getMaterialId());
                    stock.setNumber(1);
                    stock.setQuantity(stock.getQuantity());
                    return stock;
                },
                (v1, v2) -> {
                    v1.setNumber(v1.getNumber() + v2.getNumber());
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));

        // 获取库存信息
        List<Stock> stockList = listByIds(changeStockMap.keySet());
        // 库存信息
        Map<Long, Stock> stockMap = stockList.stream().collect(Collectors.toMap(Stock::getMaterialId, item -> item));

        // 循环改变物料信息
        for (Stock item : changeStockMap.values()) {
            Long materialId = item.getMaterialId();

            // 获取原库存信息
            Stock stock = stockMap.get(materialId);

            // 如果是入库
            if (typeEnum instanceof InTypeEnum) {
                if (stock != null) {
                    stock.setQuantity(stock.getQuantity().add(item.getQuantity()));
                    stock.setNumber(stock.getNumber() + item.getNumber());
                } else {
                    item.setCheckStatus(StatusConstant.NO);
                    stockList.add(item);
                }
            }
            // 如果是出库
            else {
                if (stock != null) {
                    BigDecimal quantity = stock.getQuantity().subtract(item.getQuantity());
                    stock.setQuantity(BigDecimalUtil.max(quantity, BigDecimal.ZERO));

                    int number = stock.getNumber() - item.getNumber();
                    stock.setNumber(Math.max(number, 0));
                }
            }
        }

        // 更新库存信息
        saveOrUpdateBatch(stockList);

    }

}
