package com.fjhx.stock.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.entity.stock.StockBackCorrect;
import com.fjhx.entity.stock.StockBackDetails;
import com.fjhx.params.stock.CheckDetailsVo;
import com.fjhx.params.stock.StatisticsResult;
import com.fjhx.params.stock.SubmitBackVo;
import com.fjhx.stock.service.StockBackCorrectService;
import com.fjhx.stock.service.StockBackService;
import org.springblade.core.tool.api.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 退仓面料 前端控制器
 * </p>
 *
 * @author ${author}
 * @since 2022-08-31
 */
@RestController
@RequestMapping("/stockBack")
public class StockBackController {

    @Autowired
    private StockBackService stockBackService;

    @Autowired
    private StockBackCorrectService stockBackCorrectService;

    /**
     * 获取今日领料
     */
    @PostMapping("/getPickingToday")
    public R getPickingToday(@RequestBody Map<String, String> condition) {
        List<Map<String, Object>> result = stockBackService.getPickingToday(condition);
        return R.success(result);
    }

    /**
     * 获取今日领料
     */
    @PostMapping("/getPickingTodayByUserId")
    public R getPickingTodayByUserId(@RequestBody Map<String, String> condition) {
        List<Map<String, Object>> result = stockBackService.getPickingTodayByUserId(condition);
        return R.success(result);
    }

    /**
     * 提交退料
     */
    @PostMapping("/submitBack")
    public R submitBack(@RequestBody SubmitBackVo submitBackVo) {
        stockBackService.submitBack(submitBackVo);
        return R.success();
    }

    /**
     * 退仓复核列表
     */
    @PostMapping("/checkList")
    public R checkList() {
        List<Map<String, Object>> result = stockBackService.checkList();
        return R.success(result);
    }

    /**
     * 删除
     */
    @PostMapping("/delete")
    public R delete(@RequestBody Map<String, Long> condition) {
        Long stockBackId = condition.get("stockBackId");
        stockBackService.delete(stockBackId);
        return R.success();
    }

    /**
     * 退仓复核详情
     */
    @PostMapping("/checkDetails")
    public R checkDetails(@RequestBody Map<String, Long> condition) {
        Long stockBackId = condition.get("stockBackId");
        CheckDetailsVo result = stockBackService.checkDetails(stockBackId);
        return R.success(result);
    }

    /**
     * 提交退仓复核信息
     */
    @PostMapping("/submitCheck")
    public R submitCheckDetails(@RequestBody CheckDetailsVo checkDetailsVo) {
        List<StockBackDetails> stockBackDetailsList = stockBackService.submitCheckDetails(checkDetailsVo);
        return R.success(stockBackDetailsList);
    }

    /**
     * 面料修正提交
     */
    @PostMapping("/correctSave")
    public R correctSave(@RequestBody StockBackCorrect stockBackCorrect) {
        stockBackCorrect.setCreateTime(new Date());
        stockBackCorrectService.save(stockBackCorrect);
        return R.success();
    }

    /**
     * 面料使用率用户统计
     */
    @PostMapping("/userStatistics")
    public R userStatistics(@RequestBody Map<String, String> condition) {
        Map<String, List<Map<String, Object>>> result = stockBackService.userStatistics(condition);
        return R.success(result);
    }

    /**
     * 面料使用率用户统计
     */
    @PostMapping("/statisticsDetails")
    public R statisticsDetails(@RequestBody Map<String, String> condition) {
        StatisticsResult result = stockBackService.statisticsDetails(condition);
        return R.success(result);
    }

    /**
     * 面料使用率用户统计
     */
    @PostMapping("/statisticsPage")
    public R statisticsPage(@RequestBody Map<String, String> condition) {
        Page<StatisticsResult> result = stockBackService.statisticsPage(condition);
        return R.success(result);
    }

    /**
     * 面料退仓看板用户列表
     */
    @PostMapping("/getBackUser")
    public R getBackUser(@RequestBody Map<String, String> condition) {
        Map<String, List<Map<String, Object>>> result = stockBackService.getUser(condition);
        return R.success(result);
    }

    /**
     * 面料退仓看板统计
     */
    @PostMapping("/getBackStatistics")
    public R getBackStatistics(@RequestBody Map<String, String> condition) {
        Map<String, Object> result = stockBackService.getBackStatistics(condition);
        return R.success(result);
    }


    /**
     * 面料退仓看板统计
     */
    @PostMapping("/getBackPage")
    public R getBackPage(@RequestBody Map<String, String> condition) {
        Page<Map<String, Object>> result = stockBackService.getBackPage(condition);
        return R.success(result);
    }


}

